package org.uoc.examlist.controller;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.uoc.examlist.data.ExamenDao;
import org.uoc.examlist.data.UsuarioDao;
import org.uoc.examlist.model.AlumExam;
import org.uoc.examlist.model.Examen;
import org.uoc.examlist.model.ProfExam;
import org.uoc.examlist.security.CustomAuthenticationProvider;
import org.uoc.examlist.utils.Utils;
import org.uoc.examlist.view.Calificacion;
import org.uoc.examlist.view.Calificaciones;

/**
 * Clase controladora encargada de recibir las peticiones
 * relativas a la información de las calificaciones de la aplicación web.
 * 
 * @author      Roberto Calvo Mendoza
 * @version     1.0
 * @since       1.0
 */
@Controller
@RequestMapping(value = "/calificaciones")
public class CalificacionesController {	

	@Autowired
	private ExamenDao examenDao;
	
	@Autowired
	UsuarioDao usuarioDao;	
	
	/** 
	 * Método encargado de mostrar la página con el 
	 * listado de los alumnos y sus calificaciones.
	 *
	 * @param idExamen  código del exámen
	 * @param model     objeto modelo del framework Spring 
	 * @return          nombre de la página jsp de respuesta
	 * @since           1.0
	 */		
    @RequestMapping(value = "/{idExamen}", method = RequestMethod.GET)
    public String buscadorExamen(@PathVariable("idExamen")int idExamen, Model model) {	    	
    	Calificaciones calificaciones = null;

    	
    	
    	//Recuperar las calificaciones para el idExamen
    	List<AlumExam> listAlumExam =  examenDao.findAlumExamByIdExamen(idExamen);
    	calificaciones = obtenerListaCalificaciones(listAlumExam);    	
    	calificaciones.setIdExamen(idExamen);
    	if(listAlumExam==null || listAlumExam.size()==0){
    		calificaciones.setSinAlumnos(true);
    	}    
    	
    	//Recupera datos de la asignatura
    	Examen examen = examenDao.findExamen(idExamen);
    	calificaciones.setNombreAsignatura(examen.getAsignatura().getNombre());
    	calificaciones.setFechaExamen(Utils.formatFecha(examen.getFechaHoraInicio()));
    	
    	model.addAttribute("calificaciones", calificaciones);
        return "calificaciones";
    }
    
    
    
	/** 
	 * Método encargado de actualizar las calificaciones
	 * de los alumnos que han realizado un examen.
	 *
	 * @param model     	 objeto modelo del framework Spring
	 * @param calificaciones objeto calificaciones
	 * @return          	 nombre de la página jsp de respuesta
	 * @since           	 1.0
	 */	    
    @RequestMapping(value = "/actualizar", method = RequestMethod.POST)
    public String detalleExamen(@ModelAttribute Calificaciones calificaciones, Model model) {
    	//Comprobar que el profesor tiene permiso sobre ese examen    	
		//recuperar el IdDel profesor
		String usuarioProfesor = CustomAuthenticationProvider.getUserName();
		int idUsuarioProfesor = usuarioDao.findUsuarioByLogin(usuarioProfesor).getId();
		int idProfesor = usuarioDao.findProfesor(idUsuarioProfesor).getId();  
		ProfExam profExam = examenDao.findProfExamByIdExamenIdProfesor(calificaciones.getIdExamen(), idProfesor);
		int idExamen = calificaciones.getIdExamen();
		
		if(profExam==null || profExam.getProfesor().getId()!=idProfesor){
        	model.addAttribute("error", true);   		
    	} else {
    		//Actualizar las calificaciones
    		try {
        		List<AlumExam> listaAlumExam = examenDao.updateCalificaciones(calificaciones);
        		if(listaAlumExam!=null && listaAlumExam.size()>0) {
                	calificaciones = obtenerListaCalificaciones(listaAlumExam); 
                	calificaciones.setIdExamen(idExamen);
                	model.addAttribute("ok", true);
        		}else {
                	model.addAttribute("error", true);
        		}
    		} catch(Exception ex){
        		//Si ha ido mal
            	model.addAttribute("error", true);
    		} 	
    	}
		
    	//Recupera datos de la asignatura
    	Examen examen = examenDao.findExamen(idExamen);
    	calificaciones.setNombreAsignatura(examen.getAsignatura().getNombre());
    	calificaciones.setFechaExamen(Utils.formatFecha(examen.getFechaHoraInicio()));
    	model.addAttribute("calificaciones", calificaciones);
        return "calificaciones";
    }
    
    
    private Calificaciones obtenerListaCalificaciones(List<AlumExam> listAlumExam) {
    	Calificaciones calificaciones = new Calificaciones();
    	List<Calificacion> listaCalificaciones = new ArrayList<>();    	
    	for(AlumExam alumExam :  listAlumExam) {
    		Calificacion calificacion = new Calificacion();
    		calificacion.setIdAlumno(alumExam.getAlumno().getId());
    		calificacion.setNombreCompletoAlumno(alumExam.getAlumno().getNombre() + " " + alumExam.getAlumno().getApellido1() + " " +alumExam.getAlumno().getApellido2() );
    		calificacion.setCalificacion(alumExam.getCalificacion());
    		listaCalificaciones.add(calificacion);    		
    	}
    	calificaciones.setCalificaciones(listaCalificaciones);    	
    	return calificaciones;    	
    }
    
     

}
