package org.uoc.examlist.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.uoc.examlist.data.ExamenDao;
import org.uoc.examlist.data.UsuarioDao;
import org.uoc.examlist.model.ExamLugar;
import org.uoc.examlist.model.Examen;
import org.uoc.examlist.security.CustomAuthenticationProvider;
import org.uoc.examlist.utils.Utils;
import org.uoc.examlist.view.BuscadorExamenes;
import org.uoc.examlist.view.DetalleExamen;


/**
 * Clase controladora encargada de recibir las peticiones
 * relativas a la información de los examenes de la aplicación web.
 * 
 * @author      Roberto Calvo Mendoza
 * @version     1.0
 * @since       1.0
 */
@Controller
@RequestMapping(value = "/examen")
public class ExamenController {
	
	
	
	@Autowired
	private ExamenDao examenDao;
	
	@Autowired
	UsuarioDao usuarioDao;
	
	/** 
	 * Método por defecto encargado de mostrar la página con el 
	 * buscador de asignaturas.
	 *
	 * @param model     objeto modelo del framework Spring
	 * @return          nombre de la página jsp de respuesta
	 * @since           1.0
	 */	
    @RequestMapping( method = { RequestMethod.GET, RequestMethod.POST })
    public String buscadorExamen(Model model) {	  
    	BuscadorExamenes buscadorExamenes = new BuscadorExamenes();
    	model.addAttribute("buscadorExamenes", buscadorExamenes);
        return "buscador_exam";
    }
    
	/** 
	 * Método encargado de realizar la búsqueda
	 * de exámenes.
	 *
	 * @param buscadorExamenes	objeto BuscadorExamenes
	 * @param model     		objeto modelo del framework Spring
	 * @return          		nombre de la página jsp de respuesta
	 * @since           		1.0
	 */    
    @RequestMapping(value = "/buscar", method = { RequestMethod.GET, RequestMethod.POST } )
    public String realizarBusqueda(@ModelAttribute BuscadorExamenes buscadorExamenes, Model model ) {	    	
    	String criterio = buscadorExamenes.getCriterio();
    	List<Examen> listaExamenes = null;
    	try {
    		
        	if(CustomAuthenticationProvider.getRole().equals(CustomAuthenticationProvider.ROLE_ADMIN)) {
        		listaExamenes = examenDao.findExamenesByIdAsignatura(Integer.parseInt(criterio));  	
        	} else if (CustomAuthenticationProvider.getRole().equals(CustomAuthenticationProvider.ROLE_PROF)) {
        		//recuperar el IdDel profesor
        		String usuarioProfesor = CustomAuthenticationProvider.getUserName();
        		int idUsuarioProfesor = usuarioDao.findUsuarioByLogin(usuarioProfesor).getId();
        		int idProfesor = usuarioDao.findProfesor(idUsuarioProfesor).getId();        		
        		//Buscar por idProfesor
        		listaExamenes = examenDao.findExamenesByIdAsignaturaIdProfesor(Integer.parseInt(criterio), idProfesor); 
        	} 
    		
    	}catch(NumberFormatException nfe) {
        	if(CustomAuthenticationProvider.getRole().equals(CustomAuthenticationProvider.ROLE_ADMIN)) {    		
        		listaExamenes = examenDao.findExamenesByName(criterio);
        	}else if (CustomAuthenticationProvider.getRole().equals(CustomAuthenticationProvider.ROLE_PROF)) { 
        		//recuperar el IdDel profesor
        		String usuarioProfesor = CustomAuthenticationProvider.getUserName();
        		int idUsuarioProfesor = usuarioDao.findUsuarioByLogin(usuarioProfesor).getId();
        		int idProfesor = usuarioDao.findProfesor(idUsuarioProfesor).getId(); 
        		listaExamenes = examenDao.findExamenesByNameIdProfesor(criterio, idProfesor);
        	}
    	} catch(Exception ex){
    		listaExamenes= null;
    	}    	
    	if(listaExamenes!=null) {
        	buscadorExamenes.setListaExamenes(obtenerListaDetalleExamenes(listaExamenes));
    	}

    	buscadorExamenes.setBusquedaSinResultados((buscadorExamenes.getListaExamenes() == null || buscadorExamenes.getListaExamenes().size()==0));
    	model.addAttribute("buscadorExamenes", buscadorExamenes);
        return "buscador_exam";
    }    
    
    private List<DetalleExamen> obtenerListaDetalleExamenes(List<Examen> listaExamenes) {      	
    	List<DetalleExamen> listaDetalleExamen = new ArrayList<>();    	
    	for(Examen examen: listaExamenes){
    		Set<ExamLugar> setExamLugar = examen.getExamLugars();
    		for(ExamLugar examLugar: setExamLugar) {
    			DetalleExamen detalleExamen = obtenerDetalleExamen(examLugar);    			
    			listaDetalleExamen.add(detalleExamen);
    		}
    	}
    	return listaDetalleExamen;
    }
    
    
    
    
	/** 
	 * Método por defecto encargado de mostrar la página con el 
	 * detalle de un examen	 
	 * @param idExamen  código del examen
	 * @param idLugar	código del lugar
	 * @param model     objeto modelo del framework Spring
	 * @return          nombre de la página jsp de respuesta
	 * @since           1.0
	 */	    
    @RequestMapping(value = "/detalle/{idExamen}/{idLugar}", method = RequestMethod.GET)
    public String detalleExamen(@PathVariable("idExamen")int idExamen, @PathVariable("idLugar")int idLugar, Model model) {	    	
    	ExamLugar examLugar = examenDao.findExamLugarByIdExamenIdLugar(idExamen, idLugar);    	
    	DetalleExamen detalleExamen = obtenerDetalleExamen(examLugar);    	
    	model.addAttribute("detalleExamen", detalleExamen);
    	String paginaRespuesta = null;    	
    	if(CustomAuthenticationProvider.getRole().equals(CustomAuthenticationProvider.ROLE_ADMIN)) {
    		paginaRespuesta = "detalle_exam_edit";
    	} else if (CustomAuthenticationProvider.getRole().equals(CustomAuthenticationProvider.ROLE_PROF)) {
    		paginaRespuesta = "detalle_exam_view";
    	}    	
        return paginaRespuesta;
    }
    
    private DetalleExamen obtenerDetalleExamen(ExamLugar examLugar){
    	DetalleExamen detalleExamen = new DetalleExamen();
		examLugar.getLugar().getDireccion().getDireccion();
		detalleExamen.setIdAsignatura(examLugar.getExamen().getAsignatura().getId());
		detalleExamen.setNombreAsignatura(examLugar.getExamen().getAsignatura().getNombre());		
		detalleExamen.setIdExamen(examLugar.getExamen().getId());
		detalleExamen.setIdLugar(examLugar.getLugar().getId());
		detalleExamen.setIdDireccion(examLugar.getLugar().getDireccion().getId());    			
		detalleExamen.setDireccion(examLugar.getLugar().getDireccion().getDireccion());
		detalleExamen.setEdificio(examLugar.getLugar().getEdificio());    	
		detalleExamen.setAula(examLugar.getLugar().getLocalizacion()); 
		detalleExamen.setPoblacion(examLugar.getLugar().getDireccion().getPoblacion());     	
		detalleExamen.setProvincia(examLugar.getLugar().getDireccion().getProvincia());  
		detalleExamen.setPais(examLugar.getLugar().getDireccion().getPais());  
		detalleExamen.setCodigoPostal(examLugar.getLugar().getDireccion().getCodPostal());  
		detalleExamen.setFecha(Utils.formatFecha(examLugar.getExamen().getFechaHoraInicio()));
		detalleExamen.setHora(Utils.formatHora(examLugar.getExamen().getFechaHoraInicio()));    	
		detalleExamen.setLatitud(examLugar.getLugar().getLatitud());
		detalleExamen.setLongitud(examLugar.getLugar().getLongitud());
		detalleExamen.setObservaciones(examLugar.getLugar().getComentarios());
    	return detalleExamen;    	
    }
    
    
	/** 
	 * Método encargado de actualizar los detalles 
	 * de un examen
	 *
	 * @param model     	objeto modelo del framework Spring
	 * @param detalleExamen objeto con los detalles del examen
	 * @return          	nombre de la página jsp de respuesta
	 * @since           	1.0
	 */    
    @RequestMapping(value = "/updateDetalles", method = RequestMethod.POST, produces = "text/html;charset=UTF-8")
    public String updateExamen(@ModelAttribute DetalleExamen detalleExamen, Model model ) {	   	
    	if(CustomAuthenticationProvider.getRole().equals(CustomAuthenticationProvider.ROLE_ADMIN)) {
    		ExamLugar examLugar = examenDao.updateExamLugar(detalleExamen);
	    	if(examLugar==null){
	        	model.addAttribute("error", true);   		
	    	} else {
	        	model.addAttribute("ok", true);
	    	}
    	} else {
    		model.addAttribute("error", true);   
    	}
        return "detalle_exam_edit";
    }       
    

}
