package org.uoc.examlist.controller;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.uoc.examlist.data.ExamenDao;
import org.uoc.examlist.data.UsuarioDao;
import org.uoc.examlist.model.AlumExam;
import org.uoc.examlist.model.AlumExamLugar;
import org.uoc.examlist.model.Examen;
import org.uoc.examlist.model.ProfExamLugar;
import org.uoc.examlist.model.Usuario;
import org.uoc.examlist.responsePojo.CheckAccessResponse;
import org.uoc.examlist.responsePojo.CoordenadasLugar;
import org.uoc.examlist.responsePojo.ExamenCalificacion;
import org.uoc.examlist.responsePojo.ExamenDetalle;
import org.uoc.examlist.responsePojo.ExamenItem;
import org.uoc.examlist.responsePojo.RespuestaRest;
import org.uoc.examlist.utils.SecurityUtils;
import org.uoc.examlist.utils.Utils;


/**
 * Clase controladora encargada de recibir las peticiones
 * relativas a la información de los examenes de la aplicación móvil.
 * Todas las peticiones son de tipo GET y las respuesta son en formato
 * JSon
 * 
 * @author      Roberto Calvo Mendoza
 * @version     1.0
 * @since       1.0
 */
@RestController
@RequestMapping("/rest/examen")
public class ExamenRestController {
	
	@Autowired
	private ExamenDao examenDao;
	
	@Autowired
	private UsuarioDao usuarioDao;
	

	/** 
	 * Método por defecto encargado de recibir una petición de tipo GET 
	 * para recuperar el listado de exámenes de un alumno y devolver
	 * los datos en formato json.
	 *
	 * @param request   			objeto request de la petición
	 * @param requestTimeInMillis   hora en milisegundos devuelta por el servidor para evitar peticiones repetidas. Se envía a través de la cabecera HTTP.
	 * @param idUsuario   			código del usuario. Se envía a través de la cabecera HTTP.
	 * @param auth   				token de seguridad generado en el servidor. Se envía a través de la cabecera HTTP.
	 * @return          			documento en formato json con el listado de exámenes de un alumno
	 * @since           			1.0
	 */		
    @RequestMapping(value = "/usuario", method = RequestMethod.GET, produces = "application/json;charset=UTF-8")
    public @ResponseBody  RespuestaRest<List<ExamenItem>> getExamenesByUsuario(HttpServletRequest request, @RequestHeader("X-date") String requestTimeInMillis, @RequestHeader("X-user") int idUsuario, @RequestHeader("X-auth") String auth) {	    	
    	
    	RespuestaRest<List<ExamenItem>> respuestaRest = new RespuestaRest<>();
    	
        CheckAccessResponse checkAccessResponse = SecurityUtils.checkRestAccess(request.getRequestURI(), requestTimeInMillis, idUsuario, auth, usuarioDao);    		
        if(!checkAccessResponse.getCodigo().equals("OK")){        	
        	respuestaRest.setCodigoRespuesta(checkAccessResponse.getCodigo());
        	respuestaRest.setMensajeRespuesta(checkAccessResponse.getMensaje());
        	return respuestaRest;        	
        }
    	
        //TODO controlar que el perfil pueda ser tanto de alumno como de profesor
        getExamenesByUsuario(idUsuario, respuestaRest);
        return respuestaRest;
        
    }
    
    
    private void getExamenesByUsuario(int idUsuario, RespuestaRest<List<ExamenItem>> respuestaRest) {	
    	
    	//Se recupera el perfil de usuario
    	Usuario usuario = usuarioDao.findUsuario(idUsuario);
    	List<Examen> resultadoQuery = null;
    	if("P".equals(usuario.getPerfil())) {
    		resultadoQuery = examenDao.findExamenesByIdUsuarioProfesor(idUsuario); 
    	} else if("A".equals(usuario.getPerfil())){
    		resultadoQuery = examenDao.findExamenesByIdUsuarioAlumno(idUsuario);  
    	}
    	
    	List<ExamenItem> listaExamenes = new ArrayList<>();
    	for(Examen ex: resultadoQuery) {
    		ExamenItem exItem = new ExamenItem();
    		exItem.setIdExamen(ex.getId());
    		exItem.setAsignatura(ex.getAsignatura().getNombre());
    		exItem.setFecha(Utils.formatFecha(ex.getFechaHoraInicio()));   		
    		exItem.setHoraInicio(Utils.formatHora(ex.getFechaHoraInicio()));
    		exItem.setHoraFin(Utils.formatHora(ex.getHoraFin()));       		
    		listaExamenes.add(exItem);    		
    	}
    	respuestaRest.setRespuesta(listaExamenes);
    	respuestaRest.setCodigoRespuesta("OK");  	
    }   	
    
    
	/** 
	 * Método por defecto encargado de recibir una petición de tipo GET 
	 * para recuperar el detalle de un examen. La respuesta contiene
	 * los datos en formato json.
	 *
	 * @param request   			objeto request de la petición
	 * @param requestTimeInMillis   hora en milisegundos devuelta por el servidor para evitar peticiones repetidas. Se envía a través de la cabecera HTTP.
	 * @param idUsuario   			código del usuario. Se envía a través de la cabecera HTTP.
	 * @param auth   				token de seguridad generado en el servidor. Se envía a través de la cabecera HTTP.
	 * @param idExamen 				código del exámen a mostrar.
	 * @return          			documento en formato json con el detalle del examen
	 * @since          				1.0
	 */	    
    @RequestMapping(value = "/usuario/{idExamen}", method = RequestMethod.GET, produces = "application/json;charset=UTF-8")
    public @ResponseBody RespuestaRest<ExamenDetalle> getExamenByUsuarioExamen(HttpServletRequest request, @RequestHeader("X-date") String requestTimeInMillis, @RequestHeader("X-user") int idUsuario, @RequestHeader("X-auth") String auth, @PathVariable("idExamen")int idExamen) {
    	RespuestaRest<ExamenDetalle> respuestaRest = new RespuestaRest<>();  	
    	
        CheckAccessResponse checkAccessResponse = SecurityUtils.checkRestAccess(request.getRequestURI(), requestTimeInMillis, idUsuario, auth, usuarioDao);    		
        if(!checkAccessResponse.getCodigo().equals("OK")){        	
        	respuestaRest.setCodigoRespuesta(checkAccessResponse.getCodigo());
        	respuestaRest.setMensajeRespuesta(checkAccessResponse.getMensaje());
        	return respuestaRest;        	
        }
    	

    	//Se recupera el perfil de usuario
    	Usuario usuario = usuarioDao.findUsuario(idUsuario);
    	if("P".equals(usuario.getPerfil())) { 
        	getExamenByUsuarioProfExamen(idUsuario, idExamen, respuestaRest);
    	} else if("A".equals(usuario.getPerfil())){
        	getExamenByUsuarioAlumExamen(idUsuario, idExamen, respuestaRest);    		
    	}
    	return respuestaRest;
    }
    
    private void getExamenByUsuarioAlumExamen(int idUsuarioAlum, int idExamen, RespuestaRest<ExamenDetalle> respuestaRest ) {
    	AlumExamLugar alumExamLugar = examenDao.findExamenByIdUsuarioAlumnoIdExamen(idUsuarioAlum, idExamen);
    	
    	ExamenDetalle examenDetalle = new ExamenDetalle();
    	examenDetalle.setIdExamen(alumExamLugar.getExamLugar().getExamen().getId());
    	examenDetalle.setAsignatura(alumExamLugar.getExamLugar().getExamen().getAsignatura().getNombre());
    	examenDetalle.setHoraInicio(Utils.formatHora(alumExamLugar.getExamLugar().getExamen().getFechaHoraInicio()));
    	examenDetalle.setFecha(Utils.formatFecha(alumExamLugar.getExamLugar().getExamen().getFechaHoraInicio()));
    	examenDetalle.setHoraFin(Utils.formatHora(alumExamLugar.getExamLugar().getExamen().getHoraFin()));
    	examenDetalle.setEdificio(alumExamLugar.getExamLugar().getLugar().getEdificio());
    	examenDetalle.setLocalizacion(alumExamLugar.getExamLugar().getLugar().getLocalizacion());
    	examenDetalle.setDireccion(alumExamLugar.getExamLugar().getLugar().getDireccion().getDireccion());
    	examenDetalle.setObservaciones(alumExamLugar.getExamLugar().getLugar().getComentarios());
    	CoordenadasLugar coordenadasLugar = new CoordenadasLugar();
    	coordenadasLugar.setLatitud(alumExamLugar.getExamLugar().getLugar().getLatitud());
    	coordenadasLugar.setLongitud(alumExamLugar.getExamLugar().getLugar().getLongitud());    	
    	examenDetalle.setCoordenadasLugar(coordenadasLugar);    	
    	

    	respuestaRest.setCodigoRespuesta("OK");
    	respuestaRest.setRespuesta(examenDetalle);
    }
    
    
    private void getExamenByUsuarioProfExamen(int idUsuarioProf, int idExamen, RespuestaRest<ExamenDetalle> respuestaRest ) {
    	ProfExamLugar profExamLugar = examenDao.findExamenByIdUsuarioProfesorIdExamen(idUsuarioProf, idExamen);
    	
    	ExamenDetalle examenDetalle = new ExamenDetalle();
    	examenDetalle.setIdExamen(profExamLugar.getExamLugar().getExamen().getId());
    	examenDetalle.setAsignatura(profExamLugar.getExamLugar().getExamen().getAsignatura().getNombre());
    	examenDetalle.setHoraInicio(Utils.formatHora(profExamLugar.getExamLugar().getExamen().getFechaHoraInicio()));
    	examenDetalle.setFecha(Utils.formatFecha(profExamLugar.getExamLugar().getExamen().getFechaHoraInicio()));
    	examenDetalle.setHoraFin(Utils.formatHora(profExamLugar.getExamLugar().getExamen().getHoraFin()));
    	examenDetalle.setEdificio(profExamLugar.getExamLugar().getLugar().getEdificio());
    	examenDetalle.setLocalizacion(profExamLugar.getExamLugar().getLugar().getLocalizacion());
    	examenDetalle.setDireccion(profExamLugar.getExamLugar().getLugar().getDireccion().getDireccion());
    	examenDetalle.setObservaciones(profExamLugar.getExamLugar().getLugar().getComentarios());
    	CoordenadasLugar coordenadasLugar = new CoordenadasLugar();
    	coordenadasLugar.setLatitud(profExamLugar.getExamLugar().getLugar().getLatitud());
    	coordenadasLugar.setLongitud(profExamLugar.getExamLugar().getLugar().getLongitud());    	
    	examenDetalle.setCoordenadasLugar(coordenadasLugar);    	
    	

    	respuestaRest.setCodigoRespuesta("OK");
    	respuestaRest.setRespuesta(examenDetalle);
    }
    
    
	/** 
	 * Método por defecto encargado de recibir una petición de tipo GET 
	 * para recuperar el listado de calificaciones de un alumno y devolver
	 * los datos en formato json.
	 *
	 * @param request   			objeto request de la petición
	 * @param requestTimeInMillis   hora en milisegundos devuelta por el servidor para evitar peticiones repetidas. Se envía a través de la cabecera HTTP.
	 * @param idUsuario   			código del usuario. Se envía a través de la cabecera HTTP.
	 * @param auth   				token de seguridad generado en el servidor. Se envía a través de la cabecera HTTP.
	 * @return          			documento en formato json con las últimas calificaciones de un alumno
	 * @since           			1.0
	 */    
    @RequestMapping(value = "/alumno/calificaciones", method = RequestMethod.GET, produces = "application/json;charset=UTF-8")
    public @ResponseBody RespuestaRest<List<ExamenCalificacion>> getUltimasCalificacionesByAlumno(HttpServletRequest request, @RequestHeader("X-date") String requestTimeInMillis, @RequestHeader("X-user") int idUsuario, @RequestHeader("X-auth") String auth) {	    	
    	
    	RespuestaRest<List<ExamenCalificacion>> respuestaRest = new RespuestaRest<>();

        CheckAccessResponse checkAccessResponse = SecurityUtils.checkRestAccess(request.getRequestURI(), requestTimeInMillis, idUsuario, auth, usuarioDao);    		
        if(!checkAccessResponse.getCodigo().equals("OK")){        	
        	respuestaRest.setCodigoRespuesta(checkAccessResponse.getCodigo());
        	respuestaRest.setMensajeRespuesta(checkAccessResponse.getMensaje());
        	return respuestaRest;        	
        }

    	
    	List<AlumExam> resultadoQuery = examenDao.findUltimasCalificacionesByIdAlumno(idUsuario);    	
    	List<ExamenCalificacion> listaCalificaciones = new ArrayList<>();
    	for(AlumExam alumExam: resultadoQuery){
    		ExamenCalificacion examenCalif = new ExamenCalificacion();
    		examenCalif.setIdExamen(alumExam.getExamen().getId());
    		examenCalif.setAsignatura(alumExam.getExamen().getAsignatura().getNombre());
    		examenCalif.setFecha(Utils.formatFecha(alumExam.getExamen().getFechaHoraInicio()));
    		examenCalif.setCalificacion(Utils.formatCalificacion(alumExam.getCalificacion()));
    		listaCalificaciones.add(examenCalif);    		
    	}
    	
    	respuestaRest.setRespuesta(listaCalificaciones);
    	respuestaRest.setCodigoRespuesta("OK");
    	return respuestaRest;    	
    }
    
}
