package org.uoc.examlist.controller;

import javax.persistence.NoResultException;
import javax.servlet.http.HttpServletRequest;
import javax.xml.bind.DatatypeConverter;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.uoc.examlist.data.UsuarioDao;
import org.uoc.examlist.model.Usuario;
import org.uoc.examlist.responsePojo.CheckAccessResponse;
import org.uoc.examlist.responsePojo.LoginResponse;
import org.uoc.examlist.responsePojo.RespuestaRest;
import org.uoc.examlist.responsePojo.ServerTime;
import org.uoc.examlist.utils.SecurityUtils;


/**
 * Clase controladora encargada de recibir las peticiones
 * relativas al login de la aplicación móvil.
 * Todas las peticiones son de tipo GET y las respuesta son en formato
 * JSon
 * 
 * @author      Roberto Calvo Mendoza
 * @version     1.0
 * @since       1.0
 */
@RestController
@RequestMapping("/rest/usuario")
public class UsuarioRestController {
	
	@Autowired
	UsuarioDao usuarioDao;
	
	/** 
	 * Método encargado de recibir una petición de tipo GET 
	 * para validar la combinación de usuario y contraseña de un usuario.
	 *
	 * @param login   	usuario de login.
	 * @param password	contraseña del login
	 * @return          documento en formato json indicando si el login es correcto o un código en caso de error:
	 * 					<ul>
	 * 					<li>ERR01: Combinación incorrecta de usuario/password</li>
	 * 					<li>ERR02: Error en el sistema</li>
	 * 					</ul>
	 * @since           1.0
	 */		
    @RequestMapping(value = "/login/{login}/{password}", method = RequestMethod.GET, produces = "application/json;charset=UTF-8")
    public @ResponseBody RespuestaRest<LoginResponse> login(@PathVariable("login")String login, @PathVariable("password")String password){       	
    	
    	RespuestaRest<LoginResponse> respuestaRest = new RespuestaRest<>();
    	LoginResponse loginResponse = new LoginResponse();
    	try{
    		/*
    		if(true) {
            	byte[] salt = SecurityUtils.generateSalt(); 		
            	String stSalt = DatatypeConverter.printBase64Binary(salt);           	
            	byte[] passwordEncriptado = SecurityUtils.getEncryptedPassword("test2", salt);
            	String stPasswordEncriptado = DatatypeConverter.printBase64Binary(passwordEncriptado);            	
            	System.out.println("stSalt -->" + stSalt + "<--");
            	System.out.println("stPassword -->" + stPasswordEncriptado + "<--");
    		}

        	*/
        	Usuario usuario = usuarioDao.findUsuarioByLogin(login);        	
        	byte[] salt = DatatypeConverter.parseBase64Binary(usuario.getSalt());
        	byte[] passwordEncriptado = SecurityUtils.getEncryptedPassword(password, salt);
        	String stPasswordEncriptado = DatatypeConverter.printBase64Binary(passwordEncriptado);        	
        	if(!usuario.getPassword().equals(stPasswordEncriptado)){
        		//Password incorrecto
        		respuestaRest.setCodigoRespuesta("ERR01");
        		respuestaRest.setMensajeRespuesta("Combinación incorrecta de usuario/password");      		
        	} else {
        		//Login-password correcto    
        		usuario = usuarioDao.updateToken(usuario.getId(), SecurityUtils.generarToken(usuario.getId()));
        		loginResponse.setIdUsuario(usuario.getId());
        		loginResponse.setToken(usuario.getToken());
        		loginResponse.setPerfil(usuario.getPerfil());
        		respuestaRest.setRespuesta(loginResponse);
        		respuestaRest.setCodigoRespuesta("OK");
        		respuestaRest.setMensajeRespuesta("");         		
        	}  	
    	} catch(NoResultException noResultEx) {
    		//Usuario no existe en BBDD
    		respuestaRest.setCodigoRespuesta("ERR01");
    		respuestaRest.setMensajeRespuesta("Combinación incorrecta de usuario/password");
    	}
    	catch(Exception ex) {
    		respuestaRest.setCodigoRespuesta("ERR02");
    		respuestaRest.setMensajeRespuesta("Error en el sistema");    		
    	}
    	return respuestaRest;    	
    }
    
    
    
	/** 
	 * Método encargado de recibir una petición de tipo GET 
	 * para realizar el logout de un usuario.
	 *
	 * @param request   			objeto request de la petición
	 * @param requestTimeInMillis   hora en milisegundos devuelta por el servidor para evitar peticiones repetidas. Se envía a través de la cabecera HTTP.
	 * @param idUsuario   			código del usuario. Se envía a través de la cabecera HTTP.
	 * @param auth   				token de seguridad generado en el servidor. Se envía a través de la cabecera HTTP.
	 * @return          			documento en formato json indicando si el logout es correcto
	 * @since           			1.0
	 */	
    @RequestMapping(value = "/logout", method = RequestMethod.GET, produces = "application/json;charset=UTF-8")
    public @ResponseBody RespuestaRest<LoginResponse> logout(HttpServletRequest request, @RequestHeader("X-date") String requestTimeInMillis, @RequestHeader("X-user") int idUsuario, @RequestHeader("X-auth") String auth){    	
    	
    	RespuestaRest<LoginResponse> respuestaRest = new RespuestaRest<>();
    	
        CheckAccessResponse checkAccessResponse = SecurityUtils.checkRestAccess(request.getRequestURI(), requestTimeInMillis, idUsuario, auth, usuarioDao);    		
        if(!checkAccessResponse.getCodigo().equals("OK")){        	
        	respuestaRest.setCodigoRespuesta(checkAccessResponse.getCodigo());
        	respuestaRest.setMensajeRespuesta(checkAccessResponse.getMensaje());
        	return respuestaRest;        	
        }    	
    	
    	usuarioDao.updateToken(idUsuario, "");   

    	LoginResponse loginResponse = new LoginResponse();		
    	loginResponse.setToken("");
    	respuestaRest.setRespuesta(loginResponse);
    	respuestaRest.setCodigoRespuesta("OK");
		respuestaRest.setMensajeRespuesta("");
    	return respuestaRest;
    }
    
    
	/** 
	 * Método por defecto encargado de recibir una petición de tipo GET 
	 * para obtener la hora del sistema en milisegundos. La respuesta se devuelve
	 * en formato Json
	 *
	 * @param request   objeto request de la petición
	 * @return          documento en formato json con la hora del sistema en milisegundos
	 * @since           1.0
	 */	   
    
    @RequestMapping(value = "/time", method = RequestMethod.GET, produces = "application/json;charset=UTF-8")
    public @ResponseBody RespuestaRest<String> getTime(HttpServletRequest request){

    	RespuestaRest<String> respuestaRest = new RespuestaRest<>();    	
    	respuestaRest.setCodigoRespuesta("OK");
    	respuestaRest.setRespuesta(String.valueOf(System.currentTimeMillis()));
    	return respuestaRest;
    } 

}
