package org.uoc.examlist.data;

import java.util.List;

import org.uoc.examlist.model.AlumExam;
import org.uoc.examlist.model.AlumExamLugar;
import org.uoc.examlist.model.ExamLugar;
import org.uoc.examlist.model.Examen;
import org.uoc.examlist.model.ProfExam;
import org.uoc.examlist.model.ProfExamLugar;
import org.uoc.examlist.view.Calificaciones;
import org.uoc.examlist.view.DetalleExamen;

/**
 * Interfaz con los accesos a base de datos para las consultas
 * relativas a los exámenes.
 * 
 * @author      Roberto Calvo Mendoza
 * @version     1.0
 * @since       1.0
 */
public interface ExamenDao {
	
	
	/** 
	 * Método encargado de devolver una lista de exámenes a partir 
	 * del código de usuario de un alumno.
	 *
	 * @param idUsuario	codigo de usuario de un alumno.
	 * @return          lista de exámenes
	 * @since           1.0
	 */	
	public List<Examen> findExamenesByIdUsuarioAlumno(int idUsuario);
	
	
	/** 
	 * Método encargado de devolver una lista de exámenes a partir 
	 * del código de usuario de un profesor.
	 *
	 * @param idUsuario	codigo de usuario de un profesor.
	 * @return          lista de exámenes
	 * @since           1.0
	 */	
	public List<Examen> findExamenesByIdUsuarioProfesor(int idUsuario);
	
	/** 
	 * Método encargado de devolver una lista de exámenes a partir 
	 * de parte del nombre de la asignatura.
	 *
	 * @param nombre	criterio de busqueda para comparar con el nombre de la asignatura.
	 * @return          lista de exámenes
	 * @since           1.0
	 */	
    public List<Examen> findExamenesByName(String nombre);
    
    
	/** 
	 * Método encargado de devolver una lista de exámenes a partir 
	 * de parte del nombre de la asignatura y el código del profesor
	 *
	 * @param nombre		criterio de busqueda para comparar con el nombre de la asignatura.
	 * @param idProfesor	código del profesor
	 * @return          	lista de exámenes
	 * @since           	1.0
	 */	
    public List<Examen> findExamenesByNameIdProfesor(String nombre, int idProfesor);
    
    
	/** 
	 * Método encargado de devolver una lista de exámenes a partir 
	 * del código de una asignatura
	 *
	 * @param idAsignatura	código de la asignatura.
	 * @return          	lista de exámenes
	 * @since           	1.0
	 */	    
    public List<Examen> findExamenesByIdAsignatura(int idAsignatura);    
    
    
	/** 
	 * Método encargado de devolver una lista de exámenes a partir 
	 * del código de una asignatura y el código del profesor
	 *
	 * @param idAsignatura	código de la asignatura.
	 * @param idProfesor	código del profesor
	 * @return          	lista de exámenes
	 * @since           	1.0
	 */	    
    public List<Examen> findExamenesByIdAsignaturaIdProfesor(int idAsignatura, int idProfesor);    
       
    
    
    
	/** 
	 * Método encargado de devolver una lista de objetos AlumExam a partir 
	 * del código de un examen
	 *
	 * @param idExamen		código del examen
	 * @return          	lista de objetos AlumExam
	 * @since           	1.0
	 */    
    public List<AlumExam> findAlumExamByIdExamen(int idExamen);
	
    
	/** 
	 * Método encargado de devolver un objeto ProfExam a partir 
	 * del código de examen y del código del profesor
	 *
	 * @param idExamen		código del examen
	 * @param idProfesor	código del profesor 
	 * @return          	Objeto ProfExam
	 * @since           	1.0
	 */    
    public ProfExam findProfExamByIdExamenIdProfesor (int idExamen, int idProfesor);
    
	/** 
	 * Método encargado de devolver un objeto AlumExamLugar a partir 
	 * del código de usuario del alumno y el código de examen
	 *
	 * @param idUsuario		código de usuario de un alumno
	 * @param idExamen		código del examen
	 * @return          	objeto AlumExamLugar
	 * @since           	1.0
	 */      
	public AlumExamLugar findExamenByIdUsuarioAlumnoIdExamen(int idUsuario, int idExamen);
	
	
	/** 
	 * Método encargado de devolver un objeto ProfExamLugar a partir 
	 * del código de usuario del profesor y el código de examen
	 *
	 * @param idUsuario		código de usuario de un profesor
	 * @param idExamen		código del examen
	 * @return          	objeto AlumExamLugar
	 * @since           	1.0
	 */      
	public ProfExamLugar findExamenByIdUsuarioProfesorIdExamen(int idUsuario, int idExamen);	
	
	
	/** 
	 * Método encargado de devolver una lista de objetos AlumExam a partir 
	 * del código de alumno.
	 *
	 * @param idAlumno		código del alumno
	 * @return          	lista de objetos AlumExam
	 * @since           	1.0
	 */   	
	public List<AlumExam> findUltimasCalificacionesByIdAlumno(int idAlumno);
	
	
	/** 
	 * Método encargado de devolver un objeto ExamLugar a partir 
	 * del código de examen y del código del lugar
	 *
	 * @param idExamen		código de examen
	 * @param idLugar		código de lugar
	 * @return          	objeto Examen
	 * @since           	1.0
	 */      
	public ExamLugar findExamLugarByIdExamenIdLugar(int idExamen, int idLugar);	
	
	
	
	/** 
	 * Método encargado de actualizar los datos relativos a un  
	 * examen y su lugar de realización
	 *
	 * @param detalleExamen	objeto DetalleExamen
	 * @return          	objeto ExamLugar
	 * @since           	1.0
	 */      
	public ExamLugar updateExamLugar(DetalleExamen detalleExamen);	
	
	
	/** 
	 * Método encargado de actualizar la calificación de   
	 * un alumno en un examen
	 *
	 * @param calificaciones	objeto Calificaciones
	 * @return          		Lista ExamLugar
	 * @since           		1.0
	 */      
	public List<AlumExam> updateCalificaciones(Calificaciones calificaciones);	
	
	
	
	/** 
	 * Método encargado de devolver un objeto examen
	 * a partir de su código
	 *
	 * @param idExamen		Código de examen
	 * @return          	objeto Examen
	 * @since           	1.0
	 */	
    public Examen findExamen(int idExamen);  
	
}
