package org.uoc.examlist.security;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.xml.bind.DatatypeConverter;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.uoc.examlist.data.UsuarioDao;
import org.uoc.examlist.model.Usuario;
import org.uoc.examlist.utils.SecurityUtils;

/**
 * Clase que implementa el interfaz AuthenticationProvider del framework
 * de Spring para realizar la utenticación de usuario.
 * 
 * @author      Roberto Calvo Mendoza
 * @version     1.0
 * @since       1.0
 */
@Component
public class CustomAuthenticationProvider implements AuthenticationProvider {
	
	public static final String ROLE_ADMIN = "ROLE_ADMIN";
	public static final String ROLE_PROF = "ROLE_PROF";
	
	@Autowired
	UsuarioDao usuarioDao;
	
	
	/** 
	 * Método encargado de realizar la autenticación del usuario
	 *
	 * @param authentication	objeto authentication, contiene el usuario y la contraseña
	 * @return          		objeto <code>Authentication</code> si se autentica con éxito,
	 * 							<code>null</code> en caso contrario.
	 * @since           		1.0
	 */  
    @Override
    public Authentication authenticate(Authentication authentication) throws AuthenticationException {
        String loginName = authentication.getName().toString();
        String password = authentication.getCredentials().toString();
        Usuario usuario = null;
    	try {
        	usuario = usuarioDao.findUsuarioByLogin(loginName);  
    	} catch(Exception ex) {
    		return null;    		
    	}
    	
    	String stPasswordEncriptado = "";
    	byte[] salt = DatatypeConverter.parseBase64Binary(usuario.getSalt());
    	try {
        	byte[] passwordEncriptado = SecurityUtils.getEncryptedPassword(password, salt);
        	stPasswordEncriptado = DatatypeConverter.printBase64Binary(passwordEncriptado); 
    	} catch(Exception ex) {
    		return null;
    	}       
        
        if (stPasswordEncriptado.equals(usuario.getPassword())) {
            List<GrantedAuthority> grantedAuths = new ArrayList<>();
            if("P".equals(usuario.getPerfil()) ) {
                grantedAuths.add(new SimpleGrantedAuthority(ROLE_PROF));
            } else if("D".equals(usuario.getPerfil())) {
                grantedAuths.add(new SimpleGrantedAuthority(ROLE_ADMIN));            	
            } else {
            	return null;
            }
            Authentication auth = new UsernamePasswordAuthenticationToken(loginName, password, grantedAuths);
            return auth;
        } else {
            return null;
        }
    }
 
    @Override
    public boolean supports(Class<?> authentication) {
        return authentication.equals(UsernamePasswordAuthenticationToken.class);
    }
    
    
	/** 
	 * Método encargado de devolver nombre del usuario actual
	 * @return	Nombre del usuario actual
	 * @since	1.0
	 */     
    public static String getUserName() {
    	return SecurityContextHolder.getContext().getAuthentication().getName();    	
    }
    
    
	/** 
	 * Método encargado de devolver el role o permisos del usuario actual
	 * @return	El role del usuario actual. Puede ser "ROLE_ADMIN" o "ROLE_PROF"
	 * @since   1.0
	 */    
    public static String getRole(){    	
    	String role = null;
    	Set<String> permissions = new HashSet<>();
    	for (GrantedAuthority auth : SecurityContextHolder.getContext().getAuthentication().getAuthorities()) {
    	    permissions.add(auth.getAuthority());
    	}
    	if (permissions.contains(ROLE_ADMIN)) {
    		role=ROLE_ADMIN;
    	} else if(permissions.contains(ROLE_PROF)) {    		
    		role=ROLE_PROF;
    	}
    	return role;
    }

}
