package org.uoc.examlist.utils;

import java.security.AccessControlException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.KeySpec;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.logging.Logger;

import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.xml.bind.DatatypeConverter;

import org.springframework.beans.factory.annotation.Autowired;
import org.uoc.examlist.data.UsuarioDao;
import org.uoc.examlist.model.Usuario;
import org.uoc.examlist.responsePojo.CheckAccessResponse;
import org.uoc.examlist.responsePojo.ExamenCalificacion;
import org.uoc.examlist.responsePojo.RespuestaRest;


/**
 * Clase de utilidades para seguridad con métodos relacionados con 
 * cifrado y criptografía.
 * 
 * @author      Roberto Calvo Mendoza
 * @version     1.0
 * @since       1.0
 */
public class SecurityUtils {

	/** 
	 * Método encargado de cifrar el password con una <code>salt</code> 
	 *
	 * @param password	password que se va a encriptar con el algoritmo PBKDF2WithHmacSHA1
	 * @param salt		array de bytes con la salt
	 * @return          array de bytes con la password encriptada
	 * @throws 			java.security.NoSuchAlgorithmException si no encuentra el algoritmo PBKDF2WithHmacSHA1
	 * @throws 			java.security.spec.InvalidKeySpecException  con clave erronea
	 * @since           1.0
	 */  	
	 public static byte[] getEncryptedPassword(String password, byte[] salt)
			   throws NoSuchAlgorithmException, InvalidKeySpecException {
		  // PBKDF2 with SHA-1 as the hashing algorithm. Note that the NIST
		  // specifically names SHA-1 as an acceptable hashing algorithm for PBKDF2
		  String algorithm = "PBKDF2WithHmacSHA1";
		  // SHA-1 generates 160 bit hashes, so that's what makes sense here
		  int derivedKeyLength = 160;
		  // Pick an iteration count that works for you. The NIST recommends at
		  // least 1,000 iterations:
		  // http://csrc.nist.gov/publications/nistpubs/800-132/nist-sp800-132.pdf
		  // iOS 4.x reportedly uses 10,000:
		  // http://blog.crackpassword.com/2010/09/smartphone-forensics-cracking-blackberry-backup-passwords/
		  int iterations = 20000;	
		  KeySpec spec = new PBEKeySpec(password.toCharArray(), salt, iterations, derivedKeyLength);	
		  SecretKeyFactory f = SecretKeyFactory.getInstance(algorithm);	
		  return f.generateSecret(spec).getEncoded();
	}

		
	/** 
	 * Método para generar una <code>salt</code> de forma aleatoria, con el método SHA1PRNG  
	 *
	 * @return          array de bytes con la <code>salt</code>
	 * @throws 			java.security.NoSuchAlgorithmException si no encuenra el algoritmo SHA1PRNG
	 * @since           1.0
	 */ 	 
	public static byte[] generateSalt() throws NoSuchAlgorithmException {
	   // VERY important to use SecureRandom instead of just Random
	   SecureRandom random = SecureRandom.getInstance("SHA1PRNG");
	   // Generate a 8 byte (64 bit) salt as recommended by RSA PKCS5
	   byte[] salt = new byte[8];
	   random.nextBytes(salt);	   
	   return salt;
	}
	
	
	/** 
	 * Método para generar una <code>token</code> a partir del idUsuario.
	 *
	 * @param idUsuario	código del usuario
	 * @return          <code>token</code> de seguridad
	 * @since           1.0
	 */ 
	public static String generarToken(int idUsuario) {
		return UUID.randomUUID().toString().toUpperCase() + "-" + idUsuario + "-" + System.currentTimeMillis();	
	}


	
	/** 
	 * Método para que devuelve la función <code>SHA-256</code> de un texto.
	 *
	 * @param texto		texto del que se quiere obtener la función <code>SHA-256</code>
	 * @return          resultado de calcular la función <code>SHA-256</code>
	 * @since           1.0
	 */ 
	public static String getSHA256(String texto) {
		try {
			MessageDigest md = MessageDigest.getInstance("SHA-256");
			md.update(texto.getBytes("UTF-8")); 
			byte[] digest = md.digest();
			return convertByteArrayToHexString(digest);
		} catch(Exception ex) {
			return null;
		}
	}
	
	
	private static String convertByteArrayToHexString(byte[] arrayBytes) {
	    StringBuffer stringBuffer = new StringBuffer();
	    for (int i = 0; i < arrayBytes.length; i++) {
	        stringBuffer.append(Integer.toString((arrayBytes[i] & 0xff) + 0x100, 16)
	                .substring(1));
	    }
	    return stringBuffer.toString();
	}
	
	
	/** 
	 * Método que comprueba los permisos a una petición REST.
	 *
	 * @param requestURI			URI a la que se pretende acceder.
	 * @param requestTimeInMillis	hora del servidor en milisegundos, cuando se realizó la petición original.
	 * @param idUsuario				codigo del usuario
	 * @param auth					cadena de seguridad
	 * @param usuarioDao			objeto UsuarioDao para el acceso a bbdd
	 * 
	 * @return          objeto indicando el código de acceso: OK si es correcto, 403 si no está permitido
	 * @since           1.0
	 */	
	public static CheckAccessResponse checkRestAccess(String requestURI, String requestTimeInMillis, int idUsuario, String auth, UsuarioDao usuarioDao) {
		CheckAccessResponse checkAccessReponse = new CheckAccessResponse();
		
		//Comprobamos que el usuario tenga session y no haya caducado
		try {
			checkSession(idUsuario, usuarioDao, checkAccessReponse);
		} catch(AccessControlException ace) {
			return checkAccessReponse;
		}	
		
		//Comprobamos que la petición es de hace menos de 2 minutos
		try {
			checkTime(requestTimeInMillis, checkAccessReponse);
		} catch(AccessControlException ace) {
			return checkAccessReponse;
		}
		
		//Comprobamos que la cadena de seguridad
		try {
			checkSecurityString(requestURI, requestTimeInMillis, idUsuario, auth, usuarioDao, checkAccessReponse);		
		} catch(AccessControlException ace) {
			return checkAccessReponse;
		}
		
		checkAccessReponse.setCodigo("OK");
		usuarioDao.updateAccessTime(idUsuario);
		
		return checkAccessReponse;
	}

	
	private static void checkSession(int idUsuario, UsuarioDao usuarioDao, CheckAccessResponse checkAccessReponse) {		
		Usuario usuario = usuarioDao.findUsuario(idUsuario);
		Date fechaUltimoAcceso = usuario.getFechaUltAcc();
		long now = System.currentTimeMillis();
		if(usuario.getToken()==null || usuario.getToken().equals("") || (now - fechaUltimoAcceso.getTime() > 900000)){
			usuarioDao.updateToken(idUsuario, "");
			//La sesion caduca a los 15 minutos de inactividad
			checkAccessReponse.setCodigo("403");
			checkAccessReponse.setMensaje("Unauthorized: session out-of-date ");
			throw new AccessControlException("403 Forbidden: unauthorized");			
		}		
	}
	
	private static void checkTime(String requestTimeInMillis, CheckAccessResponse checkAccessReponse) {
		long now = System.currentTimeMillis();
		if(now-Long.parseLong(requestTimeInMillis)>120000){
			checkAccessReponse.setCodigo("403");
			checkAccessReponse.setMensaje("Time Error ");
			throw new AccessControlException("403 Forbidden: time validation error");
		}		
	}
	
	private static void checkSecurityString(String requestURI, String timeInMillis, int idUsuario, String auth, UsuarioDao usuarioDao, CheckAccessResponse checkAccessReponse) {				
		requestURI = requestURI.replaceFirst("/examlist/rest/", "");  
		requestURI = requestURI.replaceFirst("/rest/", "");  
		Usuario usuario = usuarioDao.findUsuario(idUsuario);
		String token = usuario.getToken();		
		String securityString = requestURI+"|"+timeInMillis+"|"+idUsuario+"|"+token;

		if(!auth.equals(getSHA256(securityString))){
			checkAccessReponse.setCodigo("403");
			checkAccessReponse.setMensaje("Unauthorized");
			throw new AccessControlException("403 Forbidden: unauthorized");
		}
	}
	
	public static void testEncriptarPassword(String password) throws Exception {	
		byte[] salt = generateSalt();
		String stSalt = DatatypeConverter.printBase64Binary(salt);	   
		System.out.println("Salt:"+stSalt+"<--");
		getEncryptedPassword(password, salt);	   
	   	byte[] passwordEncriptado = SecurityUtils.getEncryptedPassword(password, salt);
	   	String stPasswordEncriptado = DatatypeConverter.printBase64Binary(passwordEncriptado);   
		System.out.println("stPasswordEncriptado:"+stPasswordEncriptado+"<--"); 
		
	}
	
	public static void main(String [ ] args) throws Exception {		
		testEncriptarPassword("admin1");
	}
}
